@@ -4,7 +4,7 @@ from random import choice
 
 from .path import find_path_for_agents, find_shortest_path, \
                   single_source_shortest_path
-from .consts import MAX_LABEL_COST
+from .consts import MAX_LABEL_COST, SMALL_DIVISOR
 
 
 __all__ = ['UI']
@@ -12,12 +12,12 @@ __all__ = ['UI']
 
 class Node:
 
-    def __init__(self, node_seq_no, external_node_id, zone_id, x='', y=''):
+    def __init__(self, node_seq_no, node_id, zone_id, x='', y=''):
         """ the attributes of node  """
-        # external_node_id: user defined node id from input
-        self.node_seq_no = node_seq_no
         # node_seq_no: internal node index used for calculation
-        self.external_node_id = external_node_id
+        self.node_seq_no = node_seq_no
+        # node_id: user defined node id from input
+        self.node_id = node_id
         self.outgoing_link_list = []
         self.incoming_link_list = []
         self.zone_id = zone_id
@@ -34,7 +34,7 @@ class Node:
         return self.zone_id
 
     def get_node_id(self):
-        return self.external_node_id
+        return self.node_id
 
     def get_node_no(self):
         return self.node_seq_no
@@ -48,6 +48,10 @@ class Node:
     def add_incoming_link(self, link):
         self.incoming_link_list.append(link)
 
+    def update_coordinate(self, args):
+        self.coord_x = args[0]
+        self.coord_y = args[1]
+
 
 class Link:
 
@@ -72,8 +76,8 @@ class Link:
         self.link_seq_no = link_seq_no
         self.from_node_seq_no = from_node_no
         self.to_node_seq_no = to_node_no
-        self.external_from_node = from_node_id
-        self.external_to_node = to_node_id
+        self.from_node_id = from_node_id
+        self.to_node_id = to_node_id
         # length is mile or km
         self.length = length
         self.lanes = lanes
@@ -81,7 +85,7 @@ class Link:
         self.type = link_type
         # length:km, free_speed: km/h
         self.free_flow_travel_time_in_min = (
-            length / max(0.001, free_speed) * 60
+            length / max(SMALL_DIVISOR, free_speed) * 60
         )
         # capacity is lane capacity per hour
         self.link_capacity = capacity * lanes
@@ -113,10 +117,10 @@ class Link:
         return self.link_seq_no
 
     def get_from_node_id(self):
-        return self.external_from_node
+        return self.from_node_id
 
     def get_to_node_id(self):
-        return self.external_to_node
+        return self.to_node_id
 
     def get_length(self):
         return self.length
@@ -142,13 +146,20 @@ class Link:
     def get_period_voc(self, tau):
         return self.vdfperiods[tau].get_voc()
 
+    def get_period_fftt(self, tau):
+        try:
+            return self.vdfperiods[tau].get_fftt()
+        except IndexError:
+            raise Exception(f'NO such demand period id: {tau}!'
+                            ' Check your input demand_period_id')
+
     def get_period_avg_travel_time(self, tau):
         return self.vdfperiods[tau].get_avg_travel_time()
 
     def get_generalized_cost(self, tau, value_of_time):
         return (
             self.travel_time_by_period[tau]
-            + self.toll / max(0.001, value_of_time) * 60
+            + self.toll / max(SMALL_DIVISOR, value_of_time) * 60
         )
 
     def reset_period_flow_vol(self, tau):
@@ -245,10 +256,10 @@ class Network:
         self.node_size = 0
         self.link_size = 0
         self.agent_size = 0
-        # key: external node id, value:internal node id
-        self.internal_node_seq_no_dict = {}
-        # key: internal node id, value:external node id
-        self.external_node_id_dict = {}
+        # key: node id, value: node seq no
+        self.node_id_to_no_dict = {}
+        # key: node seq no, value: node id
+        self.node_no_to_id_dict = {}
         # map link id to link seq no
         self.link_id_dict = {}
         # td:time-dependent, key:simulation time interval,
@@ -453,7 +464,7 @@ class Network:
         path = ''
         if agent.node_path:
             path = ';'.join(
-                str(self.external_node_id_dict[x]) for x in reversed(agent.node_path)
+                str(self.node_no_to_id_dict[x]) for x in reversed(agent.node_path)
             )
 
         if path_only:
@@ -506,7 +517,7 @@ class Network:
         return self.zone_to_nodes_dict[zone_id]
 
     def get_node_no(self, node_id):
-        return self.internal_node_seq_no_dict[node_id]
+        return self.node_id_to_no_dict[node_id]
 
     def get_node_size(self):
         return self.node_size
@@ -788,9 +799,12 @@ class VDFPeriod:
     def get_voc(self):
         return self.voc
 
+    def get_fftt(self):
+        return self.fftt
+
     def run_bpr(self, vol):
         vol = max(0, vol)
-        self.voc = vol / max(0.00001, self.capacity)
+        self.voc = vol / max(SMALL_DIVISOR, self.capacity)
 
         self.marginal_base = (
             self.fftt
@@ -939,8 +953,8 @@ class AccessNetwork(Network):
         self.base = base
         self.node_list = self.base.get_nodes()
         self.link_list = self.base.get_links()
-        self.map_id_to_no = self.base.internal_node_seq_no_dict
-        self.map_no_to_id = self.base.external_node_id_dict
+        self.map_id_to_no = self.base.node_id_to_no_dict
+        self.map_no_to_id = self.base.node_no_to_id_dict
         self.node_size = base.get_node_size()
         self.link_size = base.get_link_size()
         self.centroids = []
@@ -966,6 +980,8 @@ class AccessNetwork(Network):
             # create a centroid
             node_id = 'c_' + str(z)
             centroid = Node(node_seq_no, node_id, z)
+            centroid.update_coordinate(self._get_zone_coord(z))
+
             self.node_list.append(centroid)
             self.centroids.append(centroid)
 
@@ -1022,13 +1038,19 @@ class AccessNetwork(Network):
     def get_nodes_from_zone(self, zone_id):
         return self.base.zone_to_nodes_dict[zone_id]
 
+    def _get_zone_coord(self, zone_id):
+        """ coordinate of each zone is from its first node """
+        node_id = self.get_nodes_from_zone(zone_id)[0]
+        node_seq_no = self.base.get_node_no(node_id)
+        node = self.base.get_nodes()[node_seq_no]
+        return node.coord_x, node.coord_y
+
     def set_target_mode(self, mode):
         """ set up the target mode for accessibility evaluation
 
         Parameters
         ----------
-        mode : please choose one of the following three, 'p', 'w', and 'b',
-        which are in settings.yml.
+        mode : agent name which is in settings.yml.
         """
         # assert(mode in ['p', 'w', 'b', 'a'])
         self.agent_type_name = mode
@@ -1096,25 +1118,50 @@ class AccessNetwork(Network):
         link_no = self.link_predecessor[self.get_node_no(node_id)]
         return self.link_list[link_no].get_link_id()
 
-    def update_generalized_link_cost(self, at):
+    def get_sp_distance(self, node_no):
+        """ get the shortest path distance """
+        dist = 0
+        while node_no >= 0:
+            link_seq_no = self.link_predecessor[node_no]
+            if link_seq_no >= 0:
+                dist += self.get_link(link_seq_no).get_length()
+
+            node_no = self.node_predecessor[node_no]
+
+        return dist
+
+    def update_generalized_link_cost(self, at, time_dependent, demand_period_id):
         """ update generalized link costs to calculate accessibility """
         vot = at.get_vot()
-        ffs = at.get_free_flow_speed()
 
-        if at.get_type_str().startswith('p'):
+        if time_dependent:
             for link in self.get_links():
+                # do not update connectors
+                if link.get_link_id().startswith('conn_'):
+                    continue
+
                 self.link_cost_array[link.get_seq_no()] = (
-                    link.get_free_flow_travel_time()
+                    link.get_period_fftt(demand_period_id)
                     + link.get_route_choice_cost()
-                    + link.get_toll() / min(0.001, vot) * 60
+                    + link.get_toll() / min(SMALL_DIVISOR, vot) * 60
                 )
         else:
-            for link in self.get_links():
-                self.link_cost_array[link.get_seq_no()] = (
-                    (link.get_length() / max(0.001, ffs) * 60)
-                    + link.get_route_choice_cost()
-                    + link.get_toll() / min(0.001, vot) * 60
-                )
+            if at.get_type_str().startswith('p'):
+                for link in self.get_links():
+                    self.link_cost_array[link.get_seq_no()] = (
+                        link.get_free_flow_travel_time()
+                        + link.get_route_choice_cost()
+                        + link.get_toll() / min(SMALL_DIVISOR, vot) * 60
+                    )
+            else:
+                ffs = at.get_free_flow_speed()
+
+                for link in self.get_links():
+                    self.link_cost_array[link.get_seq_no()] = (
+                        (link.get_length() / max(SMALL_DIVISOR, ffs) * 60)
+                        + link.get_route_choice_cost()
+                        + link.get_toll() / min(SMALL_DIVISOR, vot) * 60
+                    )
 
 
 class Assignment:
@@ -1353,8 +1400,9 @@ class Assignment:
     def get_node_label_cost(self, node_no):
         return self.accessnetwork.get_node_label_cost(node_no)
 
-    def get_accessible_nodes(self, source_node_id, time_budget, mode):
-        if source_node_id not in self.network.internal_node_seq_no_dict.keys():
+    def get_accessible_nodes(self, source_node_id, time_budget,
+                             mode, time_dependent, tau):
+        if source_node_id not in self.network.node_id_to_no_dict.keys():
             raise Exception(f"Node ID: {source_node_id} not in the network")
 
         assert(time_budget>=0)
@@ -1375,7 +1423,9 @@ class Assignment:
         if self.accessnetwork.agent_type_name != at_name:
             self.accessnetwork.set_target_mode(at_name)
             at = self.get_agent_type(at_str)
-            self.accessnetwork.update_generalized_link_cost(at)
+            self.accessnetwork.update_generalized_link_cost(at,
+                                                            time_dependent,
+                                                            tau)
             run_sp = True
 
         if run_sp:
@@ -1394,9 +1444,11 @@ class Assignment:
 
         return nodes
 
-    def get_accessible_links(self, source_node_id, time_budget, mode):
+    def get_accessible_links(self, source_node_id, time_budget,
+                             mode, time_dependent, tau):
         # node id's
-        nodes = self.get_accessible_nodes(source_node_id, time_budget, mode)
+        nodes = self.get_accessible_nodes(source_node_id, time_budget,
+                                          mode, time_dependent, tau)
         # convert to link id's
         return [
             self.accessnetwork.get_pred_link_id(x) for x in nodes
@@ -1439,16 +1491,28 @@ class UI:
         """ find and set up shortest path for each agent """
         return self._base_assignment.find_path_for_agents(mode)
 
-    def find_shortest_path(self, from_node_id, to_node_id, mode='all', seq_type='node'):
+    def find_shortest_path(self, from_node_id, to_node_id,
+                           mode='all', seq_type='node'):
         """ return shortest path between from_node_id and to_node_id
 
         Parameters
         ----------
-        from_node_id: the starting node id
-        to_node_id  : the ending node id
-        seq_type    : 'node' or 'link'. You will get the shortest path in
-                      sequence of either node IDs or link IDs. The default is
-                      'node'
+        from_node_id
+            the starting node id
+
+        to_node_id
+            the ending node id
+
+        seq_type
+            'node' or 'link'. You will get the shortest path in sequence of
+            either node IDs or link IDs. The default is 'node'.
+
+        mode
+            the target transportation mode which is defined in settings.yml. It
+            can be either agent type or its name. For example, 'w' and 'walk'
+            are equivalent inputs.
+
+            The default is 'all', which means that links are open to all modes.
 
         Outputs
         -------
@@ -1464,15 +1528,43 @@ class UI:
             seq_type
         )
 
-    def get_accessible_nodes(self, source_node_id, time_budget, mode='p'):
+    def get_accessible_nodes(self,
+                             source_node_id,
+                             time_budget,
+                             mode='p',
+                             time_dependent=False,
+                             demand_period_id=0):
         """ get the accessible nodes from a node given mode and time budget
 
         Parameters
         ----------
-        source_node_id: the starting node id for evaluation
-        time_budget: the amount of time to travel in minutes
-        mode: transportation mode, please choose one of the following three, \
-              'p', 'w', and 'b', which are in settings.yml.
+        source_node_id
+            the starting node id for evaluation
+
+        time_budget
+            the amount of time to travel in minutes
+
+        mode
+            the target transportation mode which is defined in settings.yml. It
+            can be either agent type or its name. For example, 'w' and 'walk'
+            are equivalent inputs. Its default value is 'p' (i.e., mode auto).
+
+        time_dependent
+            True or False. Its default value is False.
+
+            If True, the accessibility will be evaluated using the period link
+            free-flow travel time (i.e., VDF_fftt). In other words, the
+            accessibility is time-dependent.
+
+            If False, the accessibility will be evaluated using the link length
+            and the free flow travel speed of each mode.
+
+        demand_period_id
+            The sequence number of demand period listed in demand_periods in
+            settings.yml. demand_period_id of the first demand_period is 0.
+
+            Use it with time_dependent when there are multiple demand periods.
+            Its default value is 0.
 
         Outputs
         -------
@@ -1481,22 +1573,36 @@ class UI:
         """
         nodes = self._base_assignment.get_accessible_nodes(source_node_id,
                                                            time_budget,
-                                                           mode)
+                                                           mode,
+                                                           time_dependent,
+                                                           demand_period_id)
 
         node_strs = ';'.join(str(x) for x in nodes)
 
         print(f'number of accessible nodes is {len(nodes)}')
         print(f'accessible nodes are: {node_strs}')
 
-    def get_accessible_links(self, source_node_id, time_budget, mode='p'):
+    def get_accessible_links(self,
+                             source_node_id,
+                             time_budget,
+                             mode='p',
+                             time_dependent=False,
+                             demand_period_id=0):
         """ get the accessible links from a node given mode and time budget
 
         Parameters
         ----------
-        source_node_id: the starting node id for evaluation
-        time_budget: the amount of time to travel in minutes
-        mode: transportation mode, please choose one of the following three, \
-              'p', 'w', and 'b', which are in settings.yml.
+        source_node_id
+            the starting node id for evaluation
+
+        time_budget
+            the amount of time to travel in minutes
+
+        mode
+            the target transportation mode which is defined in settings.yml. It
+            can be either agent type or its name. For example, 'w' and 'walk'
+            are equivalent inputs.
+
 
         Outputs
         -------
@@ -1505,7 +1611,9 @@ class UI:
         """
         links = self._base_assignment.get_accessible_links(source_node_id,
                                                            time_budget,
-                                                           mode)
+                                                           mode,
+                                                           time_dependent,
+                                                           demand_period_id)
 
         link_strs = ';'.join(str(x) for x in links)
 
